#if C_GRAPHICSMEDIA_LAB_GNU_GPL_C
#/*******************************************************************************
# * (C) Copyright 2003/2004 by Vladimir Vezhnevets <vvp@graphics.cs.msu.su> and *
# *                            Stanislav Soldatov  <stass@graphics.cs.msu.su>   *
# *******************************************************************************
# * $Id: interface.cpp,v 1.10 2004/01/18 16:33:39 dmoroz Exp $
# *******************************************************************************
# * 
# *******************************************************************************
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# *******************************************************************************/
#endif


#include "stdafx.h"
#include <string>
#include <vector>
#include <io.h>
#include <process.h>
#include "PlugCVS.h"

static char psOpNames[OP_REMOVE+1][32] = {"Update", "Commit", "Commit", "Checkout", "Export",  "Show log for", "Show stat for", "Show differences for","", "Add", "Remove"};

//==========================================================================

int AskConfirmation(Operation_enum in_Op, std::vector <std::string> &in_psFiles)
{
  char pcMessage[300];
  std::string sMsg;

  sprintf(pcMessage, GetMsg(MSG_PERFORM), psOpNames[in_Op], in_psFiles.size());
  sMsg = pcMessage;

  int iMax = 10;

  for (int i = 0; i < in_psFiles.size() && i < iMax; i++)  
    sMsg += in_psFiles[i] + "\n";

  if (in_psFiles.size() > iMax)
    sMsg += "...\n";

  return g_Info.Message(g_Info.ModuleNumber, FMSG_ALLINONE | FMSG_MB_OKCANCEL | FMSG_LEFTALIGN,
                 NULL, (const char * const *)&sMsg[0], 0, 0);
}


//==========================================================================

int AskTerminationConfirm(int in_iSeconds)
{
  char pcMessage[300];

  sprintf(pcMessage, GetMsg(MSG_TIMEOUT), in_iSeconds);

  return 1 - g_Info.Message(g_Info.ModuleNumber, FMSG_ALLINONE | FMSG_MB_YESNO | FMSG_WARNING,
                 NULL, (const char * const *)&pcMessage[0], 0, 0);
}

//==========================================================================

void ShowLogs(int in_iStatus, const char *in_pcLogFile, bool in_bNoAskIfOk)
{
  const char *ppItems[4];
  char pcMessage[100];

  int iFlags = 0, iRet = 0;

  ///   -  
  if (in_iStatus)
  {
    iFlags = FMSG_WARNING;
    sprintf(pcMessage, GetMsg(MSG_OPERATION_STATUS), in_iStatus);
  }
  else
  {
    sprintf(pcMessage, GetMsg(MSG_OK));
  }

  if (!in_bNoAskIfOk || in_iStatus)
  {
    /// 
    ppItems[0] = GetMsg(MSG_RESULT);
    ppItems[1] = pcMessage;
    ppItems[2] = GetMsg(MSG_VIEWLOG);
    ppItems[3] = GetMsg(MSG_OK);

    ///  
    iRet = g_Info.Message(g_Info.ModuleNumber, iFlags, NULL, ppItems, 4, 2);
  }

  ///   -  
  if (iRet == 0)
    g_Info.Viewer(in_pcLogFile, "CVS log", -1, -1, -1, -1, VF_ENABLE_F6);
}



//==========================================================================

Operation_enum ShowOperationMenu(bool bCurDirOnly)
{
  static FarMenuItem pItems[13] ={{"&Update...", 1, 0, 0}, 
                           {"&Commit", 0, 0, 0},
                           {"Ligh&t (modem) commit", 0, 0, 0},
                           {"Check&out...", 0, 0, 0},
                           {"E&xport...", 0, 0, 0},
                           {"&Log", 0, 0, 0},
                           {"&Stat", 0, 0, 0},
                           {"&Differences", 0, 0, 0},
                           {"", 0, 0, 1},
                           {"&Add", 0, 0, 0},
                           {"&Remove", 0, 0, 0},
                           {"&Browse", 0, 0, 0},
                           {"&Mark >", 0, 0, 0}},
              pItems2[4] = {{"&Modified", 1, 0, 0}, 
                            {"&Conflicts", 0, 0, 0},
                            {"&Out-of-date", 0, 0, 0},
                            {"&Not in repository", 0, 0, 0}};

  int iOperation;

  const char* title=bCurDirOnly?"CVS: Current Dir":"CVS: Selection";

  
  struct _stat finfo;
  if(_stat("CVS",&finfo)==-1 || !(finfo.st_mode&_S_IFDIR))
  {
    if(bCurDirOnly)
    {
      iOperation = g_Info.Menu(g_Info.ModuleNumber, -1, -1, 0, 
                               FMENU_AUTOHIGHLIGHT | FMENU_WRAPMODE, title, 
                             "no module here",
                             NULL, NULL, NULL, &pItems[OP_CHECKOUT], 2);
      if(iOperation >-1)
        iOperation += OP_CHECKOUT;

      return (Operation_enum)iOperation;
    }
    else
      return 
        (Operation_enum)
        g_Info.Menu(g_Info.ModuleNumber, -1, -1, 0, 
                               FMENU_AUTOHIGHLIGHT | FMENU_WRAPMODE, title, 
                             "no module here",
                             NULL, NULL, NULL, pItems, OP_DIFF+1);
  }

  while (1)
  {
    ///  
    iOperation = g_Info.Menu(g_Info.ModuleNumber, -1, -1, 0, 
                             FMENU_AUTOHIGHLIGHT | FMENU_WRAPMODE, title, 
                             NULL, NULL, NULL, NULL, pItems, OP_SHOW + 1);

    if (iOperation == OP_SHOW)
    {
      iOperation = OP_SHOW_CHANGED +
                          g_Info.Menu(g_Info.ModuleNumber, -1, -1, 0, 
                                FMENU_AUTOHIGHLIGHT | FMENU_WRAPMODE, "CVS: Mark", 
                                NULL, NULL, NULL, NULL, pItems2, 4);
      if (iOperation < OP_SHOW_CHANGED)
        continue;
    }

    break;
  }
  ///  
  return (Operation_enum) iOperation;
}

ViewOperation_enum ShowViewMenu(const char* filename,int sel_files)
{
  static FarMenuItem menu[6] =
                          {{"", 1, 0, 0}, 
                           {"", 0, 0, 0},
                           {"", 0, 0, 0},
                           {"", 0, 0, 1},
                           {"", 0, 0, 0},
                           {"", 0, 0, 0}};

  if(!filename) filename="";

  char title[80];
  sprintf(title,"View %s",filename);

  sprintf(menu[VOP_VIEW].Text,"&View %s",filename);
  sprintf(menu[VOP_LOG].Text,"&Log %s",filename);
  sprintf(menu[VOP_DIFF].Text,"&Diff %s",filename);

  if(sel_files==0)
  {
    return (ViewOperation_enum)
            g_Info.Menu(g_Info.ModuleNumber, -1, -1, 0, 
                        FMENU_WRAPMODE, title, 
                        NULL,NULL, NULL, NULL, menu, VOP_DIFF+1);
  }
  else
  {
    sprintf(menu[VOP_LOGMANY].Text,"Lo&g %d files",sel_files);
    sprintf(menu[VOP_DIFFMANY].Text,"Di&ff %d files",sel_files);
  
    return  (ViewOperation_enum)
            g_Info.Menu(g_Info.ModuleNumber, -1, -1, 0, 
                        FMENU_WRAPMODE, title, 
                        NULL,NULL, NULL, NULL, menu, VOP_DIFFMANY+1);
  }
}

//==========================================================================
/*
int ShowAddMenuParam()
{
   struct InitDialogItem InitItems []={
    DI_DOUBLEBOX,3,1,72,8,0,0,0,0,(char *)GetMsg(MSG_ADD_BINARY),
    DI_BUTTON,0,7,0,0,0,0,DIF_CENTERGROUP,1,(char *)"Ok",
    DI_BUTTON,0,7,0,0,0,0,DIF_CENTERGROUP,0,(char *)"Cancel"
  };

  struct FarDialogItem DialogItems[sizeof(InitItems) / sizeof(InitItems[0])];

  InitDialogItems(InitItems, DialogItems, sizeof(InitItems)/sizeof(InitItems[0]));

  int ExitCode = g_Info.Dialog(g_Info.ModuleNumber,
                          -1,-1,76,10,
                          "TempCfg",DialogItems,
                           sizeof(DialogItems) / sizeof(DialogItems[0]));

  if (ExitCode != 7)
    return(FALSE);
}*/


//===================================================================
//= Function name	: UpdateParam
//= Description   :    update
//= Return type   : bool 
//===================================================================

bool GetUpdateParam(std::vector <const char *> &out_ppcArgs)
{
  //  
  static FarDialogItem Dialog []=
    {
      {DI_DOUBLEBOX, 3, 1, 47, 12,0,0,0,0,0},
      {DI_BUTTON, 0, 11, 0, 0, 0, 0, DIF_CENTERGROUP, 1, (char *)"OK"},
      {DI_BUTTON, 0, 11, 0, 0, 0, 0, DIF_CENTERGROUP, 0, (char *)"Cancel"},
      {DI_RADIOBUTTON, 5, 2, 35, 0, 0, 1, DIF_GROUP, 0, (char *)"to current branch (no options)"},
      {DI_RADIOBUTTON, 5, 3, 39, 0, 0, 0, 0, 0, (char *)"to main trunk, clear all tags (-A)"},
      {DI_RADIOBUTTON, 5, 4, 36, 0, 0, 0, 0, 0, (char *)"to a specific revision/tag (-r)"},
      {DI_RADIOBUTTON, 5, 6, 36, 0, 0, 0, 0, 0, (char *)"to a revision no later than date (-D)"},
      {DI_EDIT, 6, 5, 22, 0, 0, (int)(const char*)"revision", DIF_HISTORY | DIF_SELECTONENTRY, 0, 0},
      {DI_EDIT, 6, 7, 22, 0, 0, (int)(const char*)"date", DIF_HISTORY, 0, 0},
      {DI_CHECKBOX, 5, 9, 15, 0, 0, 1, 0, 0, (char*)"R&ecursive"},
      {DI_TEXT,4, 8, 28, 8, 0, 0, DIF_SEPARATOR,0,0},
      {DI_TEXT,4, 10, 28, 10, 0, 0, DIF_SEPARATOR,0,0},
      {DI_TEXT, 27, 7, 39, 0, 0, 0, 0, 0, (char*)"(YYYY-MM-DD)"},
    };

  static char pcStorage[255];

  //  
  while(1)
  {
    int ExitCode = g_Info.Dialog(g_Info.ModuleNumber,
                                 -1, -1, 51, 14,
                                 NULL, Dialog,
                                 sizeof(Dialog) / sizeof(Dialog[0]));
    //  Esc  Cancel - 
    if(ExitCode == -1 || ExitCode == 2)
      return false;

    // ,         
    if(Dialog[5].Param.Selected && Dialog[7].Data.Data[0] == 0)
    {
      g_Info.Message(g_Info.ModuleNumber, FMSG_WARNING | FMSG_MB_OK | FMSG_ALLINONE,
                     NULL, (const char * const *)"Error\n Revision or tag is required", 0, 0);      
      continue;
    };

    if(Dialog[6].Param.Selected && Dialog[8].Data.Data[0] == 0)
    {
      g_Info.Message(g_Info.ModuleNumber, FMSG_WARNING | FMSG_MB_OK | FMSG_ALLINONE,
                     NULL, (const char * const *)"Error\n Date is required", 0, 0);      
      continue;
    };

    break;
  };


  ///     

  if (!Dialog[9].Param.Selected)
  {
    //   
    out_ppcArgs.push_back("-l");
  }

  if (Dialog[3].Param.Selected)
  {
    //  
  }
  else
  if (Dialog[4].Param.Selected)
  {
    // Main trunk
    out_ppcArgs.push_back("-A");
  }
  else
  if (Dialog[5].Param.Selected)
  {
    // Revision
    out_ppcArgs.push_back("-r");
    strncpy(pcStorage, &Dialog[7].Data.Data[0], 255);
    out_ppcArgs.push_back(pcStorage);
  }
  else
  if (Dialog[6].Param.Selected)
  {
    // Date
    out_ppcArgs.push_back("-D");
    strncpy(pcStorage, &Dialog[8].Data.Data[0], 255);
    out_ppcArgs.push_back(pcStorage);
  }

  return true;
};



